module  fft(
    clk        ,
    rst_n      ,
    en,
    fft_dout0  ,
    fft_dout1  ,
    fft_dout_vld
    );

    
    parameter          DATA_N  = 512;
    parameter          LOGN    = 9  ;
    parameter          DATA_W1 = 8  ;
    
    input              clk          ;
    input              rst_n        ;
    input              en           ;
    output[65 : 0]     fft_dout0    ;
    output[65 : 0]     fft_dout1    ;
    output             fft_dout_vld  ;

    reg                flag       ;
    
    reg[7 : 0]         cnt0       ;
    reg[7 : 0]         cnt1       ;
    reg[3 : 0]         cnt2       ;
    wire               add_cnt0   ;
    wire               end_cnt0   ;
    wire               add_cnt1   ;
    wire               end_cnt1   ;
    wire               add_cnt2   ;
    wire               end_cnt2   ;
    
    reg[7 : 0]         wr_cnt0    ;
    reg[7 : 0]         wr_cnt1    ;
    reg[3 : 0]         wr_cnt2    ;
    wire               add_wr_cnt0;
    wire               end_wr_cnt0;
    wire               add_wr_cnt1;
    wire               end_wr_cnt1;
    wire               add_wr_cnt2;
    wire               end_wr_cnt2;
    
    reg[9 : 0]         addr_0     ;
    reg[65 : 0]        wdata0     ;
    reg                wrreq0     ;
    wire[65 : 0]       q0         ;
    
    reg[9 : 0]         addr_1     ;
    reg[65: 0]         wdata1     ;
    reg                wrreq1     ;
    wire[65 : 0]       q1         ;
    
    reg[9 : 0]         addr_2     ;
    reg[65 : 0]        wdata2     ;
    reg                wrreq2     ;
    wire[65 : 0]       q2         ;

    
    reg[9 : 0]         addr_3     ;
    reg[65 : 0]        wdata3     ;
    reg                wrreq3     ;
    wire[65 : 0]       q3         ;
    
    reg[7 : 0]         addr_cos   ;
    wire[15 : 0]       qcos       ;
    
    reg                fft_din_vld;
    reg[65 : 0]        fft_din0   ;
    reg[65 : 0]        fft_din1   ;
    reg[15 : 0]        fft_cos    ;
    wire[65 : 0]       fft_dout0  ;
    wire[65 : 0]       fft_dout1  ;
    wire               fft_dout_vld;

    
    reg                flag_ff0   ;
    reg                flag_ff1   ;
    
    wire[9 : 0]        rd_addr0   ;
    wire[9 : 0]        rd_addr1   ;
    wire[9 : 0]        wr_addr0   ;
    wire[9 : 0]        wr_addr1   ;
    wire               fir_rd     ;
    wire               fir_wr     ;
    wire               sec_rd     ;
    wire               sec_wr     ;
    
    
    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            flag <= 0;
        end
        else if(en || (end_wr_cnt1 == 1 && end_wr_cnt2 == 0)) begin //дRAMһֲҲһʱflag
            flag <= 1;
        end
        else if(end_cnt1)begin   //RAMһֶflagͣдRAMд
            flag <= 0;
        end
    end
    
   
   /*********www.mdy-edu.com ƽ עͿʼ****************
                   RAM3
   **********www.mdy-edu.com ƽ עͽ****************/

    always @(posedge clk or negedge rst_n)begin
        if(!rst_n)begin
            cnt0 <= 0;
        end
        else if(add_cnt0)begin
            if(end_cnt0)
                cnt0 <= 0;
            else
                cnt0 <= cnt0 + 1;
        end
    end
    
    assign add_cnt0 = flag == 1;
    assign end_cnt0 = add_cnt0 && cnt0 == (1 << cnt2) - 1;
    
    always @(posedge clk or negedge rst_n)begin 
        if(!rst_n)begin
            cnt1 <= 0;
        end
        else if(add_cnt1)begin
            if(end_cnt1)
                cnt1 <= 0;
            else
                cnt1 <= cnt1 + 1;
        end
    end
    
    assign add_cnt1 = end_cnt0;
    assign end_cnt1 = add_cnt1 && cnt1 == (DATA_N >> (cnt2 + 1)) - 1;
    
    always @(posedge clk or negedge rst_n)begin
        if(!rst_n)begin
            cnt2 <= 0;
        end
        else if(add_cnt2)begin
            if(end_cnt2)
                cnt2 <= 0;
            else
                cnt2 <= cnt2 + 1;
        end
    end
    
    assign add_cnt2 = end_cnt1;
    assign end_cnt2 = add_cnt2 && cnt2 == LOGN - 1;
     
 
/*********www.mdy-edu.com ƽ עͿʼ****************
                 дRAM3
**********www.mdy-edu.com ƽ עͽ****************/

    always @(posedge clk or negedge rst_n)begin
        if(!rst_n)begin
            wr_cnt0 <= 0;
        end
        else if(add_wr_cnt0)begin
            if(end_wr_cnt0)
                wr_cnt0 <= 0;
            else
                wr_cnt0 <= wr_cnt0 + 1;
        end
    end
    
    assign add_wr_cnt0 = fft_dout_vld;  
    assign end_wr_cnt0 = add_wr_cnt0 && wr_cnt0 == (1 << wr_cnt2) - 1;
    
    always @(posedge clk or negedge rst_n)begin 
        if(!rst_n)begin
            wr_cnt1 <= 0;
        end
        else if(add_wr_cnt1)begin
            if(end_wr_cnt1)
                wr_cnt1 <= 0;
            else
                wr_cnt1 <= wr_cnt1 + 1;
        end
    end
    
    assign add_wr_cnt1 = end_wr_cnt0;
    assign end_wr_cnt1 = add_wr_cnt1 && wr_cnt1 == (DATA_N >> (wr_cnt2 + 1)) - 1;
    
    always @(posedge clk or negedge rst_n)begin
        if(!rst_n)begin
            wr_cnt2 <= 0;
        end
        else if(add_wr_cnt2)begin
            if(end_wr_cnt2)
                wr_cnt2 <= 0;
            else
                wr_cnt2 <= wr_cnt2 + 1;
        end
    end
    
    assign add_wr_cnt2 = end_wr_cnt1;
    assign end_wr_cnt2 = add_wr_cnt2 && wr_cnt2 == LOGN - 1;
    

        assign rd_addr0 = cnt0 + (2 * cnt1 * (1 << cnt2));  //rd_addr0=cnt0+cnt1*2^(cnt2+1);
        assign rd_addr1 = rd_addr0 + (1 << cnt2);   //rd_addr1=x1+2^cnt2;
        
        assign wr_addr0 = wr_cnt0 + (2 * wr_cnt1 * (1 << wr_cnt2));  
        assign wr_addr1 = wr_addr0 +(1 << wr_cnt2); 
        
        /*********www.mdy-edu.com ƽ עͿʼ****************
          дRAM1λǵڼ
        **********www.mdy-edu.com ƽ עͽ****************/

        assign fir_rd = wr_cnt2[0] == 0;     
        assign fir_wr = wr_cnt2[0] == 1;
        
        assign sec_rd = wr_cnt2[0] == 1; 
        assign sec_wr = wr_cnt2[0] == 0; 
         
        /*********www.mdy-edu.com ƽ עͿʼ****************
         һ˫RAMַдַдʹ
        **********www.mdy-edu.com ƽ עͽ****************/

        always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            addr_0 <= 0;
        end
        else if(fir_rd) begin
            addr_0 <= rd_addr0;
        end
        else begin
            addr_0 <= wr_addr0;
        end
    end
  

    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            wdata0 <= 0;
        end
        else begin
            wdata0 <= fft_dout0;
        end
    end
   
    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            wrreq0 <= 0;
        end
        else if(fir_wr) begin
            wrreq0 <= fft_dout_vld;
        end
        else begin
            wrreq0 <= 0;
        end
    end
    
  
    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            addr_1 <= 0;
        end
        else if(fir_rd) begin
            addr_1 <= rd_addr1;
        end
        else begin
            addr_1 <= wr_addr1;
        end
    end
   
    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            wdata1 <= 0;
        end
        else begin
            wdata1 <= fft_dout1;
        end
    end

    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            wrreq1 <= 0;
        end
        else if(fir_wr) begin
            wrreq1 <= fft_dout_vld;
        end
        else begin
            wrreq1 <= 0;
        end
    end
    
   
  
   /*********www.mdy-edu.com ƽ עͿʼ****************
         ڶ˫RAMַдַдʹ
   **********www.mdy-edu.com ƽ עͽ****************/
    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            addr_2 <= 0;
        end
        else if(sec_rd) begin
            addr_2 <= rd_addr0;
        end
        else begin
            addr_2 <= wr_addr0;
        end
    end
   
    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            wdata2 <= 0;
        end
        else begin
            wdata2 <= fft_dout0;
        end
    end
    
    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            wrreq2 <= 0;
        end
        else if(sec_wr) begin
            wrreq2 <= fft_dout_vld;
        end
        else begin
            wrreq2 <= 0;
        end
    end
     
    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            addr_3 <= 0;
        end
        else if(sec_rd) begin
            addr_3 <= rd_addr1;
        end
        else begin
            addr_3 <= wr_addr1;
        end
    end
   
    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            wdata3 <= 0;
        end
        else begin
            wdata3 <= fft_dout1;
        end
    end
    

    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            wrreq3 <= 0;
        end
        else if(sec_wr) begin
            wrreq3 <= fft_dout_vld;
        end
        else begin
            wrreq3 <= 0;
        end
    end

  always  @(posedge clk or negedge rst_n)begin     //3ģʹfft_din_vldfft_dinӦ
        if(rst_n==1'b0)begin
            flag_ff0 <= 0;
            flag_ff1 <= 0;
            fft_din_vld <= 0;
        end
        else begin
            flag_ff0 <= flag;
            flag_ff1 <= flag_ff0;
            fft_din_vld <= flag_ff1;
        end
    end
   
    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            fft_din0 <= 0;
        end
        else if(fir_rd) begin
            fft_din0 <= q0;
        end
        else begin
            fft_din0 <= q2;
        end
    end
    //fft_din1
    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            fft_din1 <= 0;
        end
        else if(fir_rd) begin
            fft_din1 <= q1;
        end
        else begin
            fft_din1 <= q3;
        end
    end
    //fft_cos
    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            fft_cos <= 0;
        end
        else begin
            fft_cos <= qcos;
        end
    end
   
    always  @(posedge clk or negedge rst_n)begin
        if(rst_n==1'b0)begin
            addr_cos <= 0;
        end
        else begin
            addr_cos <= cnt0 * (DATA_N >> (cnt2 + 1))*(512/DATA_N);
        end
    end
    
     ram_2x_1 dataram(
        .address_a(addr_0    ),
        .address_b(addr_1    ),
        .clock    (clk       ),
        .data_a   (wdata0    ),
        .data_b   (wdata1    ),
        .wren_a   (wrreq0    ),
        .wren_b   (wrreq1    ),
        .q_a      (q0        ),
        .q_b      (q1        )  
        );
    //WNK
    wnk_rom wnk(
        .address(addr_cos     ),
        .clock  (clk          ),
        .q      (qcos         )
        );
   
    fushuadd fushu(
        .clk        (clk         ),
        .rst_n      (rst_n       ),
        .fft_din_vld(fft_din_vld ),
        .din0       (fft_din0    ),
        .din1       (fft_din1    ),
        .fft_cos    (fft_cos     ),
        .dout_vld   (fft_dout_vld),
        .douta      (fft_dout0   ),
        .doutb      (fft_dout1   )
     );
    
    ram_2x_2 ram(
        .address_a(addr_2    ), 
        .address_b(addr_3    ), 
        .clock    (clk       ), 
        .data_a   (wdata2    ), 
        .data_b   (wdata3    ), 
        .wren_a   (wrreq2    ), 
        .wren_b   (wrreq3    ), 
        .q_a      (q2        ), 
        .q_b      (q3        )
        );
endmodule
